import React from "react";
import TodoList from "./TodoList";
import TodoHeader from "./TodoHeader";
import InputField from "./InputField";
import $ from "jquery";

const _createTodo = (todos, title) => {
    todos.push({
        id: todos.length ? todos[todos.length - 1].id + 1 : 1,
        title,
        completed: false
    });
    return todos;
};

const _updateTodo = (todos, id, title) => {
    const target = todos.find((todo) => todo.id === id);
    if (target) target.title = title;
    return todos;
};

const _toggleTodo = (todos, id, completed) => {
    const target = todos.find((todo) => todo.id === id);
    if (target) target.completed = completed;
    return todos;
};

const _deleteTodo = (todos, id) => {
    const idx = todos.findIndex((todo) => todo.id === id);
    if (idx !== -1) todos.splice(idx, 1);
    return todos;
};

export default class TodoApp extends React.Component {
    constructor(props, context) {
        super(props, context);
        this.state = {
            todos: []
        };
    }

    componentDidMount() {
        $.ajax({
                url: '/todos.json',
                type: 'GET',
                success: (data) => {
                    this.setState({todos: data});
                },
                error: (error) => {
                    console.log(error);
                },
            }
        );
    }

    updateTodosBy(updateFn) {
        return (...args) => {
            this.setState({
                todos: updateFn(this.state.todos, ...args)
            });
        };
    }

    render() {
        const { todos } = this.state;
        return (
            <div>
                <TodoHeader
                    title="我的待辦清單"
                    username="Jason"
                    todoCount={todos.filter((todo) => !todo.completed).length}
                />
                <InputField
                    placeholder="新增待辦清單"
                    onSubmitEditing={this.updateTodosBy(_createTodo)}
                />
                <TodoList
                    todos={todos}
                    onUpdateTodo={this.updateTodosBy(_updateTodo)}
                    onToggleTodo={this.updateTodosBy(_toggleTodo)}
                    onDeleteTodo={this.updateTodosBy(_deleteTodo)}
                />
            </div>
        );
    }
}