import React from "react";
import InputField from "./InputField";
import styles from "./TodoItem.css";

export default class TodoItem extends React.Component {
    constructor(props, context) {
        super(props, context);
        this.state = { editable: false };
        this.toggleEditMode = this.toggleEditMode.bind(this);
    }

    toggleEditMode() {
        this.setState({ editable: !this.state.editable });
    }

    renderViewMode() {
        const {
            title,
            completed,
            onToggle,
            onDelete
        } = this.props;

        // 注意这是jsx语法，不能写class而应该写className
        return (
            <div className={styles.TodoItemViewDiv}>
                <input
                    type="checkbox"
                    checked={completed}
                    onChange={() => onToggle && onToggle(!completed)}
                />
                <span onDoubleClick={this.toggleEditMode}>{title}</span>
                <button onClick={() => onDelete && onDelete()}>x</button>
            </div>
        );
    }

    renderEditMode() {
        const { title, onUpdate } = this.props;
        return (
            <InputField
                autoFocus
                placeholder="編輯待辦事項"
                value={title}
                onBlur={this.toggleEditMode} // 失去焦点
                onKeyDown={(e) => { // ESC
                    if (e.keyCode === 27) {
                        e.preventDefault();
                        this.toggleEditMode();
                    }
                }}
                // 提交
                onSubmitEditing={(content) => {
                    onUpdate && onUpdate(content);
                    this.toggleEditMode();
                }}
            />
        );
    }

    render() {
        return this.state.editable ?
            this.renderEditMode() :
            this.renderViewMode();
    }
}

TodoItem.propTypes = {
    title: React.PropTypes.string.isRequired,
    completed: React.PropTypes.bool.isRequired,
    onUpdate: React.PropTypes.func,
    onToggle: React.PropTypes.func,
    onDelete: React.PropTypes.func
};